"""
Varlord setup.py

Minimal setup.py for backward compatibility.
All package metadata is defined in pyproject.toml.
"""
from setuptools import setup

# This setup.py is kept for backward compatibility.
# All configuration is in pyproject.toml.
setup()

